/*
 *      EGADS: Electronic Geometry Aircraft Design System
 *
 *             Sew 2 Bodies or sew up an IGES like input
 *
 *      Copyright 2011-2014, Massachusetts Institute of Technology
 *      Licensed under The GNU Lesser General Public License, version 2.1
 *      See http://www.opensource.org/licenses/lgpl-2.1.php
 *
 */

#include <math.h>
#include "egads.h"



int main(int argc, char *argv[])
{
  int i, j, k, mtype, oclass, nbody, nface1, nface2, *senses;
  ego context, model1, model2, newModel, geom, *bodies1, *bodies2;
  ego *faces, *faces1, *faces2;
  
  if ((argc < 5) && (argc != 2)) {
    printf("\n Usage: sew filename1 filename2 face1# face2#\n\n");
    return 1;
  }

  /* initialize */
  printf(" EG_open           = %d\n", EG_open(&context));
  printf(" EG_loadModel 1    = %d\n", EG_loadModel(context, 0, argv[1], &model1));
  if (model1 == NULL) {
    printf(" EG_close          = %d\n", EG_close(context));
    return 1;
  }
  /* get all bodies */
  printf(" EG_getTopology 1  = %d\n", EG_getTopology(model1, &geom, &oclass,
                                                     &mtype, NULL, &nbody,
                                                     &bodies1, &senses));
  faces = bodies1;
  k     = nbody;
  
  if (argc != 2) {
    /* get all faces in body */
    printf(" EG_getBodyTopos 1 = %d\n", EG_getBodyTopos(bodies1[0], NULL, FACE,
                                                        &nface1, &faces1));
  
    printf(" EG_loadModel 2    = %d\n", EG_loadModel(context, 0, argv[2], &model2));
    if (model2 == NULL) {
      EG_free(faces1);
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model1));
      printf(" EG_close          = %d\n", EG_close(context));
      return 1;
    }
    /* get all bodies */
    printf(" EG_getTopology 2  = %d\n", EG_getTopology(model2, &geom, &oclass,
                                                       &mtype, NULL, &nbody,
                                                       &bodies2, &senses));
    /* get all faces in body */
    printf(" EG_getBodyTopos 2 = %d\n", EG_getBodyTopos(bodies2[0], NULL, FACE,
                                                        &nface2, &faces2));

    faces = (ego *) malloc((nface1+nface2)*sizeof(ego));
    if (faces == NULL) {
      printf(" ERROR: malloc on %d egos!\n", nface1+nface2-2);
      EG_free(faces1);
      EG_free(faces2);
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model1));
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model2));
      printf(" EG_close          = %d\n", EG_close(context));
      return 1;
    }
    sscanf(argv[3], "%d", &j);
    if ((j < 0) || (j > nface1)) {
      printf(" ERROR: Argument 3 is %d [1-%d]!\n", j, nface1);
      free(faces);
      EG_free(faces1);
      EG_free(faces2);
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model1));
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model2));
      printf(" EG_close          = %d\n", EG_close(context));
      return 1;
    }
    for (k = i = 0; i < nface1; i++) {
      if (i == j-1) continue;
      faces[k] = faces1[i];
      k++;
    }
    sscanf(argv[4], "%d", &j);
    if ((j < 0) || (j > nface2)) {
      printf(" ERROR: Argument 4 is %d [1-%d]!\n", j, nface2);
      free(faces);
      EG_free(faces1);
      EG_free(faces2);
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model1));
      printf(" EG_deleteObject   = %d\n", EG_deleteObject(model2));
      printf(" EG_close          = %d\n", EG_close(context));
      return 1;
    }
    for (i = 0; i < nface2; i++) {
      if (i == j-1) continue;
      faces[k] = faces2[i];
      k++;
    }
  }
  
  /* merge each of them */
  printf(" EG_sewFaces       = %d\n", EG_sewFaces(k, faces, 0.0, 0, &newModel));
  if (argc != 2) {
    free(faces);
    EG_free(faces1);
    EG_free(faces2);
  }

  /* write it out */
  if (newModel != NULL) {
    printf(" EG_saveModel      = %d\n", EG_saveModel(newModel, "sew.egads"));
    printf("\n");
    printf(" EG_deleteObject   = %d\n", EG_deleteObject(newModel));
  }
  
  printf(" EG_deleteObject   = %d\n", EG_deleteObject(model1));
  if (argc != 2) printf(" EG_deleteObject   = %d\n", EG_deleteObject(model2));
  printf(" EG_close          = %d\n", EG_close(context));
  return 0;
}
